@@ -1,5 +1,8 @@
 import enum
 import typing
+import warnings
+
+import attr
 
 from uecp.commands.base import (
     UECPCommand,
@@ -42,7 +45,7 @@ class ProgrammeIdentificationSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("ProgrammeIdentificationSetCommand", int):
+    ) -> tuple["ProgrammeIdentificationSetCommand", int]:
         data = list(data)
         if len(data) < 5:
             raise UECPCommandDecodeNotEnoughData(len(data), 5)
@@ -126,7 +129,7 @@ class ProgrammeServiceNameSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("ProgrammeServiceNameSetCommand", int):
+    ) -> tuple["ProgrammeServiceNameSetCommand", int]:
         data = list(data)
         if len(data) < 11:
             raise UECPCommandDecodeNotEnoughData(len(data), 11)
@@ -194,7 +197,7 @@ class DecoderInformationSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("DecoderInformationSetCommand", int):
+    ) -> tuple["DecoderInformationSetCommand", int]:
         data = list(data)
         if len(data) < 4:
             raise UECPCommandDecodeNotEnoughData(len(data), 4)
@@ -262,7 +265,7 @@ class TrafficAnnouncementProgrammeSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("TrafficAnnouncementProgrammeSetCommand", int):
+    ) -> tuple["TrafficAnnouncementProgrammeSetCommand", int]:
         data = list(data)
         if len(data) < 4:
             raise UECPCommandDecodeNotEnoughData(len(data), 4)
@@ -356,7 +359,7 @@ class ProgrammeTypeSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("ProgrammeTypeSetCommand", int):
+    ) -> tuple["ProgrammeTypeSetCommand", int]:
         data = list(data)
         if len(data) < 4:
             raise UECPCommandDecodeNotEnoughData(len(data), 4)
@@ -433,7 +436,7 @@ class ProgrammeTypeNameSetCommand(UECPCommand, UECPCommandDSNnPSN):
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("ProgrammeTypeNameSetCommand", int):
+    ) -> tuple["ProgrammeTypeNameSetCommand", int]:
         data = list(data)
         if len(data) < 11:
             raise UECPCommandDecodeNotEnoughData(len(data), 11)
@@ -464,6 +467,57 @@ class InvalidNumberOfTransmissions(UECPCommandException):
     pass
 
 
+def _ensure_radio_text_carriage_return(*values: str) -> str:
+    if len(values) < 1:
+        import inspect
+
+        raise TypeError(
+            f"{inspect.stack()[0][3]} must be called with at least one argument and the last one must be a string"
+        )
+    value = values[-1]
+    value = str(value)
+    if 0 < len(value) < 61 and value[-1] != "\r":
+        value += "\r"
+        warnings.warn(
+            "Implicitly appending a carriage return 0x0D to radio text shorter than 61 characters"
+        )
+    return value
+
+
+def _check_radio_text(_, __, value: str):
+    value = str(value)
+    if len(value) > 64 or len(value) == 0:
+        raise ValueError(
+            f"Radio text supports only up to 64 characters and must not be empty, given {value!r}",
+        )
+    if len(value) < 61 and value[-1] != "\r":
+        raise ValueError(
+            f"Radio text shorter than 61 characters must be terminated by a carriage return, given {value!r}"
+        )
+    value.encode("basic_rds_character_set")
+
+    return value
+
+
+@attr.s
+class RadioText:
+    text: str = attr.ib(
+        converter=_ensure_radio_text_carriage_return,
+        validator=_check_radio_text,
+        on_setattr=[
+            _ensure_radio_text_carriage_return,  # type:ignore
+            _check_radio_text,
+        ],
+    )
+    number_of_transmissions: int = attr.ib(default=0, converter=int)
+    a_b_toggle: bool = attr.ib(default=False, converter=bool)
+
+    @number_of_transmissions.validator
+    def _check_number_of_transmissions(self, _, new_not: int):
+        if not (0x0 <= new_not <= 0xF):
+            raise InvalidNumberOfTransmissions(new_not)
+
+
 @UECPCommand.register_type
 class RadioTextSetCommand(UECPCommand, UECPCommandDSNnPSN):
     ELEMENT_CODE = 0x0A
@@ -477,95 +531,81 @@ class RadioTextSetCommand(UECPCommand, UECPCommandDSNnPSN):
         buffer_configuration=RadioTextBufferConfiguration.TRUNCATE_BEFORE,
         data_set_number=0,
         programme_service_number=0,
+        radiotext: RadioText = None,
     ):
         super().__init__(
             data_set_number=data_set_number,
             programme_service_number=programme_service_number,
         )
-        self.__text = ""
-        self.text = text
-        self.__number_of_transmissions = 0
-        self.number_of_transmissions = number_of_transmissions
-        self.__a_b_toggle = bool(a_b_toggle)
-        self.__buffer_configuration = RadioTextBufferConfiguration(buffer_configuration)
+        if radiotext is not None:
+            self._radiotext = radiotext
+        else:
+            self._radiotext = RadioText(
+                text=text,
+                number_of_transmissions=number_of_transmissions,
+                a_b_toggle=a_b_toggle,
+            )
+        self._buffer_configuration = RadioTextBufferConfiguration(buffer_configuration)
 
     @property
     def text(self) -> str:
-        return self.__text
+        return self._radiotext.text
 
     @text.setter
     def text(self, new_text: str):
-        new_text = str(new_text)
-        if len(new_text) > 64:
-            raise InvalidProgrammeTypeName(
-                new_text, f"Radio text supports only up to 64 characters"
-            )
-        new_text = new_text.rstrip(" ")
-        try:
-            new_text.encode("basic_rds_character_set")
-        except ValueError as e:
-            raise InvalidProgrammeServiceName(
-                new_text, f"Radio text cannot be encoded, exc={e!r}"
-            )
-        self.__text = new_text
+        self._radiotext.text = new_text
 
     @property
     def number_of_transmissions(self) -> int:
-        return self.__number_of_transmissions
+        return self._radiotext.number_of_transmissions
 
     @number_of_transmissions.setter
     def number_of_transmissions(self, new_not: int):
-        try:
-            if new_not == int(new_not):
-                new_not = int(new_not)
-            else:
-                raise ValueError()
-        except ValueError:
-            raise InvalidNumberOfTransmissions(new_not)
-
-        if not (0x0 <= new_not <= 0xF):
-            raise InvalidNumberOfTransmissions(new_not)
-        self.__number_of_transmissions = new_not
+        self._radiotext.number_of_transmissions = new_not
 
     @property
     def a_b_toggle(self) -> bool:
-        return self.__a_b_toggle
+        return self._radiotext.a_b_toggle
 
     @a_b_toggle.setter
     def a_b_toggle(self, toggle: bool):
-        self.__a_b_toggle = bool(toggle)
+        self._radiotext.a_b_toggle = toggle
 
     @property
     def buffer_configuration(self) -> RadioTextBufferConfiguration:
-        return self.__buffer_configuration
+        return self._buffer_configuration
 
     @buffer_configuration.setter
     def buffer_configuration(self, buffer_conf: RadioTextBufferConfiguration):
-        self.__buffer_configuration = RadioTextBufferConfiguration(buffer_conf)
+        self._buffer_configuration = RadioTextBufferConfiguration(buffer_conf)
+
+    @property
+    def radiotext(self) -> RadioText:
+        return self._radiotext
 
     def encode(self) -> list[int]:
         data = [self.ELEMENT_CODE, self.data_set_number, self.programme_service_number]
         if (
-            len(self.__text) == 0
-            and self.__buffer_configuration
+            len(self._radiotext.text) == 0
+            and self._buffer_configuration
             is RadioTextBufferConfiguration.TRUNCATE_BEFORE
         ):
             data.append(0)
         else:
-            mel = 1 + len(self.__text)
+            mel = 1 + len(self._radiotext.text)
             flags = (
-                self.__buffer_configuration << 5
-                | self.__number_of_transmissions << 1
-                | self.__a_b_toggle
+                self._buffer_configuration << 5
+                | self._radiotext.number_of_transmissions << 1
+                | self._radiotext.a_b_toggle
             )
             data += [mel, flags]
-            data += list(self.__text.encode("basic_rds_character_set"))
+            data += list(self._radiotext.text.encode("basic_rds_character_set"))
         return data
 
     @classmethod
     def create_from(
         cls, data: typing.Union[bytes, list[int]]
-    ) -> ("RadioTextSetCommand", int):
+    ) -> tuple["RadioTextSetCommand", int]:
         data = list(data)
         if len(data) < 4:
             raise UECPCommandDecodeNotEnoughData(len(data), 4)
@@ -573,15 +613,16 @@ class RadioTextSetCommand(UECPCommand, UECPCommandDSNnPSN):
         if mec != cls.ELEMENT_CODE:
             raise UECPCommandDecodeElementCodeMismatchError(mec, cls.ELEMENT_CODE)
         if mel == 0:
-            return cls(data_set_number=0, programme_service_number=0)
+            return cls(data_set_number=0, programme_service_number=0), 4
         data = data[4:]
         if len(data) < mel:
             raise UECPCommandDecodeNotEnoughData(len(data), mel)
         flags = data[0]
-        text = bytes(data[1:mel]).decode("basic_rds_character_set")
         buffer_configuration = (flags & 0b0110_0000) >> 5
         number_of_transmission = (flags & 0b0001_1110) >> 1
         a_b_toggle = flags & 0b0000_0001
+        text_data = bytes(data[1:mel])
+        text = text_data.decode("basic_rds_character_set")
 
         return (
             cls(
